#include "Button.h"

#include "TextNode.h"
#include "GeometryNode.h"
#include "TransformNode.h"
#include "ThemeManager.h"
#include "PaintEvent.h"

#include "FontTokens.h"

#include "MouseEvent.h"
#include "FileLogger.h"

Button::Button(ButtonData::Component component)
    : Widget(),
      mLabel(),
      mClickFunc()
{
    mStyle.mComponent = component;
    mName = "Button";

    setHeight(mStyle.getContainerHeight());
    setMaxHeight(mStyle.getContainerHeight());
    setRadius(mStyle.getContainerCornerRadius());
}

Button::~Button()
{

}

std::unique_ptr<Button> Button::Create(ButtonData::Component component)
{
    return std::make_unique<Button>(component);
}

void Button::setOnClickFunction(clickFunc func)
{
    mClickFunc = func;
}

void Button::setLabel(const std::string& text)
{
    if (text != mLabel)
    {
        mLabel = text;
        mContentDirty = true;
    }
}

void Button::setState(ButtonData::State state)
{
    if (mStyle.mState != state)
    {
        mStyle.mState = state;
        mStateDirty = true;
    }
}

void Button::setEnabled(bool isEnabled)
{
    if (mEnabled != isEnabled)
    {
        mEnabled = isEnabled;
        if (mStyle.mState == ButtonData::State::Disabled)
        {
            setState(ButtonData::State::Enabled);
        }
    }
}

void Button::updateState()
{
    setBackground(mStyle.getContainerColor());
    setBackgroundTone(mStyle.getContainerSurfaceTintColor());
    setElevation(mStyle.getContainerElevation());

    setLabelTextColor(mStyle.getLabelTextColor());
    setLabelTextOpacity(mStyle.getLabelOpacity());
    setLabelTextTypescale(mStyle.getLabelTypescale());
}

void Button::setLabelTextColor(Theme::Sys::Color color)
{
    if (mLabelTextColor != color)
    {
        mLabelTextColor = color;
        mMaterialDirty = true;
    }
}

void Button::setLabelTextOpacity(float opacity)
{
    if (mLabelOpacity != opacity)
    {
        mLabelOpacity = opacity;
        mMaterialDirty = true;
    }
}

void Button::setLabelTextTypescale(Theme::Sys::Typescale typescale)
{
    if (mLabelTextTypescale != typescale)
    {
        mLabelTextTypescale = typescale;
        mMaterialDirty = true;
    }
}

void Button::onMyMouseEvent(const MouseEvent* event)
{
    if (!mEnabled)
    {
        return;
    }

    if (event->getAction() == MouseEvent::Action::Pressed)
    {
        setState(ButtonData::State::Pressed);
    }
    else if (event->getAction() == MouseEvent::Action::Released)
    {
        resetState();
        if (mClickFunc)
        {
            mClickFunc(this);
        }
    }
    else if (event->getAction() == MouseEvent::Action::Enter)
    {
        resetState();
    }
    else if (event->getAction() == MouseEvent::Action::Leave)
    {
        resetState();
    }
}

void Button::resetState()
{
    if (mHasFocus)
    {
        setState(ButtonData::State::Focused);
    }
    else if (mContainsMouse)
    {
        setState(ButtonData::State::Hovered);
    }
    else if (mEnabled)
    {
        setState(ButtonData::State::Enabled);
    }
    else
    {
        setState(ButtonData::State::Disabled);
    }
}

bool Button::isDirty() const
{
    return Widget::isDirty() || mContentDirty;
}

void Button::doPaint(const PaintEvent* event)
{
    updateBackground(event);
    updateLabel(event);
}

void Button::updateLabel(const PaintEvent* event)
{
    if (!mTextNode)
    {
        mTextNode = TextNode::Create(mLabel, mLocation);
        mTextNode->setName(mName + "_TextNode");
        mTextNode->setContent(mLabel);
        mRootNode->addChild(mTextNode.get());
    }

    if (mTransformDirty)
    {
        mTextNode->setLocation(mLocation);
        mTextNode->setWidth(mSize.mWidth);
        mTextNode->setHeight(mSize.mHeight);
    }

    if (mMaterialDirty)
    {
        auto fill_color = event->getThemesManager()->getColor(mLabelTextColor);
        fill_color.setAlpha(mLabelOpacity);
        mTextNode->setFillColor(fill_color);
        
        auto size = FontTokens::getSize(mLabelTextTypescale);
        auto family = FontTokens::getFont(FontTokens::getFont(mLabelTextTypescale));
        auto font_data = FontItem(family, static_cast<float>(size));
        mTextNode->setFont(font_data);
    }

    if (mContentDirty)
    {
        mTextNode->setContent(mLabel);
        mContentDirty = false;
    }

    if (mVisibilityDirty)
    {
        mTextNode->setIsVisible(mVisible);
    }
}
